@@ -8,15 +8,32 @@ from abc import ABC, abstractmethod
 class DerbyGame:
     """Class for storing all the data related to a derby game.
     """
-    def __init__(self, pdf_jams_data: pd.DataFrame, game_data_dict: Dict[str, str]):
+    def __init__(self, pdf_jams_data: pd.DataFrame, game_data_dict: Dict[str, str],
+                 pdf_penalties: pd.DataFrame):
+        self.pdf_jams_data = pdf_jams_data
         self.game_data_dict = game_data_dict
         self.team_1_name = game_data_dict["team_1"]
         self.team_2_name = game_data_dict["team_2"]
-        self.pdf_jams_data = pdf_jams_data
+        self.game_summary_dict = self.extract_game_summary_dict()
+        self.n_jams = self.game_summary_dict["Jams"]
+        self.pdf_penalties = pdf_penalties
 
     def extract_game_summary(self) -> pd.DataFrame:
         """Build a gross game-summary dataframe
 
+        Returns:
+            pd.DataFrame: game summary dataframe
+        """
+        gross_summary_dict = self.extract_game_summary_dict()
+        pdf_game_summary = pd.DataFrame({
+            "Statistic": gross_summary_dict.keys(),
+            "Value": gross_summary_dict.values()})
+
+        return pdf_game_summary
+
+    def extract_game_summary_dict(self) -> pd.DataFrame:
+        """Build a gross game-summary dictionary
+
         Returns:
             pd.DataFrame: game summary dataframe
         """
@@ -34,12 +51,7 @@ class DerbyGame:
             f"{self.team_1_name} Final Score": final_score_team_1,
             f"{self.team_2_name} Final Score": final_score_team_2,
         }
-
-        pdf_game_summary = pd.DataFrame({
-            "Statistic": gross_summary_dict.keys(),
-            "Value": gross_summary_dict.values()})
-
-        return pdf_game_summary
+        return gross_summary_dict
 
     def extract_game_teams_summary(self) -> pd.DataFrame:
         """Build a summary dataframe with per-team data
@@ -72,16 +84,43 @@ class DerbyGame:
         cols_repeated_byteam = [x for x in self.pdf_jams_data.columns
                                 if x.endswith("_1")]
         cols_repeated_byteam = [x[:-2] for x in cols_repeated_byteam]
-        pdf_repeatedcols_team1 = (self.pdf_jams_data[["prd_jam"] +
-                                  [x + "_1" for x in cols_repeated_byteam]].copy())
+        pdf_repeatedcols_team1 = (
+            self.pdf_jams_data[["prd_jam", "PeriodNumber"] +
+            [x + "_1" for x in cols_repeated_byteam]].copy())
         pdf_repeatedcols_team1["team"] = self.team_1_name
         pdf_repeatedcols_team1 = pdf_repeatedcols_team1.rename(
             columns={x + "_1": x for x in cols_repeated_byteam})
         pdf_repeatedcols_team2 = self.pdf_jams_data[
-            ["prd_jam"] + [x + "_2" for x in cols_repeated_byteam]].copy()
+            ["prd_jam", "PeriodNumber"] +
+            [x + "_2" for x in cols_repeated_byteam]].copy()
         pdf_repeatedcols_team2["team"] = self.team_2_name
         pdf_repeatedcols_team2 = pdf_repeatedcols_team2.rename(
             columns={x + "_2": x for x in cols_repeated_byteam})
 
         pdf_jam_data_long = pd.concat([pdf_repeatedcols_team1, pdf_repeatedcols_team2])
-        return pdf_jam_data_long
\ No newline at end of file
+        return pdf_jam_data_long
+
+    def build_team_jammersummary_df(self, team_number: int):
+        """Build a dataframe with data on all the jammers for a team.
+
+        Args:
+            team_number (int): Team number
+        """
+        jammer_col = f"jammer_name_{team_number}"
+        jamscore_col = f"JamScore_{team_number}"
+        netpoints_col = f"net_points_{team_number}"
+        lead_col = f"Lead_{team_number}"
+
+        pdf_jammer_data = self.pdf_jams_data.groupby(jammer_col).agg({
+            jamscore_col: "sum",
+            netpoints_col: "mean",
+            'Number': "count",
+            lead_col: "mean"}).reset_index()
+        pdf_jammer_data = pdf_jammer_data.rename(columns={
+            jammer_col: "Jammer",
+            jamscore_col: "Total Score",
+            netpoints_col: "Mean Net Points",
+            lead_col: "Proportion Lead",
+            "Number": "Jams"
+        })
+        return pdf_jammer_data
\ No newline at end of file
