@@ -2,62 +2,16 @@ __author__ = "Damon May"
 
 import pandas as pd
 from matplotlib.figure import Figure
-from typing import Dict, List
+from typing import List, Optional
 import seaborn as sns
 from matplotlib import pyplot as plt
-from matplotlib.backends.backend_pdf import PdfPages
 from jamstats.data.game_data import DerbyGame
 import logging
 
+
 logger = logging.Logger(__name__)
 
-def save_game_plots_to_pdf(derby_game: DerbyGame,
-                           out_filepath: str) -> None:
-    """Read in a jams .tsv file, make all the plots, write to a .pdf
 
-    Args:
-        in_filepath (str): jams .tsv filepath
-        out_filepath (str): output .pdf filepath
-    """
-    prepare_to_plot()
-    figures = make_all_plots(derby_game)
-    pdfout = PdfPages(out_filepath)
-    logging.debug(f"Saving {len(figures)} figures to {out_filepath}")
-    for figure in figures:
-        pdfout.savefig(figure)
-    pdfout.close()
-    logging.debug(f"Wrote {out_filepath}")
-
-
-def prepare_to_plot() -> None:
-    """Prepare Seaborn to make pretty plots.
-    """
-    sns.set_context("talk")
-    sns.set_style("white")
-
-def make_all_plots(derby_game: DerbyGame) -> List[Figure]:
-    """Build all plots, suitable for exporting to a .pdf
-
-    Args:
-        derby_game (DerbyGame): a derby game
-    Returns:
-        List[Figure]: figures
-    """
-    figures = []
-    for plot_func in [
-        plot_game_summary_table,
-        plot_game_teams_summary_table,
-        plot_jam_lead_and_scores,
-        plot_cumulative_score_by_jam,
-        plot_jammers_by_team,
-        histogram_jam_duration,
-        plot_lead_summary
-    ]:
-        try:
-            figures.append(plot_func(derby_game))
-        except Exception as e:
-            logger.warn(f"Failed to make plot: {e}")
-    return figures
 
 
 def plot_jammers_by_team(derby_game: DerbyGame) -> Figure:
@@ -164,17 +118,28 @@ def plot_game_teams_summary_table(derby_game: DerbyGame) -> Figure:
             colLabels=None, bbox=[0,0,1,1])
     return f
 
+def plot_jam_lead_and_scores_period1(derby_game: DerbyGame) -> Figure:
+    return plot_jam_lead_and_scores(derby_game, period=1)
 
-def plot_jam_lead_and_scores(derby_game: DerbyGame) -> Figure:
+def plot_jam_lead_and_scores_period2(derby_game: DerbyGame) -> Figure:
+    return plot_jam_lead_and_scores(derby_game, period=2)
+
+def plot_jam_lead_and_scores(derby_game: DerbyGame,
+                             period: Optional[int] = None) -> Figure:
     """Given a long-format jam dataframe, visualize lead and scores per jam
 
     Args:
         derby_game (DerbyGame): a derby game
+        period (int): Period to plot. If not provided, plot both
 
     Returns:
         Figure: figure
     """
+    logger.debug("Plotting jam lead and scores...")
     pdf_jam_data_long = derby_game.build_jams_dataframe_long()
+    if period is not None:
+        logger.debug(f"Restricting to period {period}")
+        pdf_jam_data_long = pdf_jam_data_long[pdf_jam_data_long.PeriodNumber == period]
 
     f, (ax0, ax1) = plt.subplots(1, 2, gridspec_kw={'width_ratios': [1, 4]})
     
@@ -183,7 +148,7 @@ def plot_jam_lead_and_scores(derby_game: DerbyGame) -> Figure:
     
     ax = ax0
     pdf_jam_data_long_byjam = pdf_jam_data_long.sort_values(["prd_jam", "team_number"])
-    pdf_jambools = pdf_jam_data_long_byjam[["Lead", "Lost", "Calloff", "NoInitial", "StarPass"]]
+    pdf_jambools = pdf_jam_data_long_byjam[["Lead", "Calloff", "Lost", "NoInitial", "StarPass"]]
     team_color_map = {derby_game.team_1_name: 1,
                       derby_game.team_2_name: 2}
     team_colors = [team_color_map[team] for team in pdf_jam_data_long_byjam.team]
@@ -194,6 +159,25 @@ def plot_jam_lead_and_scores(derby_game: DerbyGame) -> Figure:
     colors = [(1,1,1), sns.color_palette()[0], sns.color_palette()[1]]
     pdf_jambool_heatmap = pd.DataFrame(jamboolint_dict)
     sns.heatmap(pdf_jambool_heatmap, ax=ax, cbar=False, cmap=sns.color_palette(colors, as_cmap=True))
+    # add lines separating jams
+    for i in range(len(pdf_jambools)):
+        if i % 2 == 0:
+            pdf_linedata = {
+                "x": [0, len(pdf_jambools.columns)],
+                "y": [i, i],
+            }
+            sns.lineplot(x="x", y="y", data=pdf_linedata, color="black", ax=ax)
+    # add letter indicators of attributes
+    for i in range(len(pdf_jambools.columns)):
+        col = pdf_jambools.columns[i]
+        vals = list(pdf_jambools[col])
+        for j in range(len(vals)):
+            if vals[j]:
+                ax.text(i + .5, j + .5, col[0], size="x-small",
+                        horizontalalignment="center",
+                        verticalalignment="center")
+    ax.set_xlabel("")
+    ax.set_ylabel("")
     ax.set_xticks([x+.5 for x in range(len(pdf_jambool_heatmap.columns))])
     ax.set_yticklabels([])
     ax.set_xticklabels(pdf_jambool_heatmap.columns, rotation=90)
@@ -201,13 +185,29 @@ def plot_jam_lead_and_scores(derby_game: DerbyGame) -> Figure:
 
     ax = ax1
     sns.barplot(x="JamScore", y="prd_jam", data=pdf_jam_data_long, hue="team", ax=ax)
+    n_period_jams = len(set(pdf_jam_data_long.prd_jam))
     ax.legend()
-    ax.set_title("Points per jam by team")
+    # add lines separating jams
+    highscore = ax.get_xlim()[1]
+    for i in range(n_period_jams):
+        pdf_linedata = {
+            "x": [0, highscore],
+            "y": [i + 0.5, i + 0.5],
+        }
+        sns.lineplot(x="x", y="y", data=pdf_linedata, color="black", ax=ax)
+    ax.set_xlim((0, highscore))
+    ax.set_ylim((n_period_jams - 0.5, -0.5))
+    title = "Points per jam by team"
+    if period is not None:
+        title = title + f" (period {period})"
+    ax.set_title(title)
     ax.set_xlabel("Points")
     ax.set_ylabel(None)
 
     f.set_size_inches(8, 11)
     f.tight_layout()
+
+    logger.debug("Done plotting.")
     return f
 
 
